/*************************************************************************
 *                     COPYRIGHT NOTICE
 *            Copyright 2019 Horizon Robotics, Inc.
 *                   All rights reserved.
 *************************************************************************/
#ifndef __HB_AUDIO_ERROR_H__
#define __HB_AUDIO_ERROR_H__

#include "hb_sys.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#define HB_ERR_APPID  (0x1000FC00L)

#define HB_DEF_ERR(module, errid) \
		((int)((HB_ERR_APPID) | (module << 16) | errid ))

typedef enum {
	EN_ERR_AIN_INVALID_PARAM = 1,
	EN_ERR_AIN_INVALID_DEVID = 2,
	EN_ERR_AIN_NOMEM = 3,
	EN_ERR_AIN_NOT_ALLOWED = 4,
	EN_ERR_AIN_INVALID_CHN = 5,
	EN_ERR_AIN_READ_BUFFER = 6,
} EN_AIN_ERR_CODE_E;

typedef enum {
	EN_ERR_AOT_INVALID_PARAM = 1,
	EN_ERR_AOT_INVALID_DEVID = 2,
	EN_ERR_AOT_NOT_ALLOWED = 3,
	EN_ERR_AOT_INVALID_CHN = 4,
	EN_RR_AOT_WRITE_BUFFER = 5,
} EN_AOT_ERR_CODE_E;

typedef enum {
	EN_ERR_AENC_INVALID_CHNID = 4,
	EN_ERR_AENC_NOTREADY = 5,
	EN_ERR_AENC_BUSY = 6,
	EN_ERR_AENC_NOT_ALLOWED = 7,
	EN_ERR_AENC_NOMEM = 8,
	EN_ERR_AENC_INVALID_BUFFER = 9,
	EN_ERR_AENC_NOT_SUPPORT = 10,
	EN_ERR_AENC_UNEXIST = 11,
} EN_AENC_ERR_CODE_E;

typedef enum {
	EN_ERR_ADEC_INVALID_CHNID = 4,
	EN_ERR_ADEC_NOTREADY = 5,
	EN_ERR_ADEC_BUSY = 6,
	EN_ERR_ADEC_NOT_ALLOWED = 7,
	EN_ERR_ADEC_NOMEM = 8,
	EN_ERR_ADEC_INVALID_BUFFER = 9,
	EN_ERR_ADEC_NOT_SUPPORT = 10,
	EN_ERR_ADEC_INVALID_PARAM = 11,
} EN_ADEC_ERR_CODE_E;

#define HB_ERR_INVALID_CHANNEL               0x1000FC01
#define HB_ERR_CHANNEL_MAP_EXIST             0x1000FC02
#define HB_ERR_CHANNEL_MAP_UNEXIST           0x1000FC03

#define HB_ERR_AENC_INVALID_CHNID            HB_DEF_ERR(HB_ID_AENC, EN_ERR_AENC_INVALID_CHNID)
#define HB_ERR_AENC_NOTREADY                 HB_DEF_ERR(HB_ID_AENC, EN_ERR_AENC_NOTREADY)
#define HB_ERR_AENC_BUSY                     HB_DEF_ERR(HB_ID_AENC, EN_ERR_AENC_BUSY)
#define HB_ERR_AENC_NOT_ALLOWED              HB_DEF_ERR(HB_ID_AENC, EN_ERR_AENC_NOT_ALLOWED)
#define HB_ERR_AENC_NOMEM                    HB_DEF_ERR(HB_ID_AENC, EN_ERR_AENC_NOMEM)
#define HB_ERR_AENC_INVALID_BUFFER           HB_DEF_ERR(HB_ID_AENC, EN_ERR_AENC_INVALID_BUFFER)
#define HB_ERR_AENC_NOT_SUPPORT              HB_DEF_ERR(HB_ID_AENC, EN_ERR_AENC_NOT_SUPPORT)
#define HB_ERR_AENC_INVALID_PARAM            HB_DEF_ERR(HB_ID_AENC, EN_ERR_AENC_UNEXIST)

#define HB_ERR_ADEC_INVALID_CHNID            HB_DEF_ERR(HB_ID_ADEC, EN_ERR_ADEC_INVALID_CHNID)
#define HB_ERR_ADEC_NOTREADY                 HB_DEF_ERR(HB_ID_ADEC, EN_ERR_ADEC_NOTREADY)
#define HB_ERR_ADEC_BUSY                     HB_DEF_ERR(HB_ID_ADEC, EN_ERR_ADEC_BUSY)
#define HB_ERR_ADEC_NOT_ALLOWED              HB_DEF_ERR(HB_ID_ADEC, EN_ERR_ADEC_NOT_ALLOWED)
#define HB_ERR_ADEC_NOMEM                    HB_DEF_ERR(HB_ID_ADEC, EN_ERR_ADEC_NOMEM)
#define HB_ERR_ADEC_INVALID_BUFFER           HB_DEF_ERR(HB_ID_ADEC, EN_ERR_ADEC_INVALID_BUFFER)
#define HB_ERR_ADEC_NOT_SUPPORT              HB_DEF_ERR(HB_ID_ADEC, EN_ERR_ADEC_NOT_SUPPORT)
#define HB_ERR_ADEC_INVALID_PARAM            HB_DEF_ERR(HB_ID_ADEC, EN_ERR_ADEC_INVALID_PARAM)

#define HB_ERR_AIN_INVALID_PARAM             HB_DEF_ERR(HB_ID_AIN, EN_ERR_AIN_INVALID_PARAM)
#define HB_ERR_AIN_INVALID_DEVID             HB_DEF_ERR(HB_ID_AIN, EN_ERR_AIN_INVALID_DEVID)
#define HB_ERR_AIN_NOMEM                     HB_DEF_ERR(HB_ID_AIN, EN_ERR_AIN_NOMEM)
#define HB_ERR_AIN_NOT_ALLOWED               HB_DEF_ERR(HB_ID_AIN, EN_ERR_AIN_NOT_ALLOWED)
#define HB_ERR_AIN_INVALID_CHN               HB_DEF_ERR(HB_ID_AIN, EN_ERR_AIN_INVALID_CHN)
#define HB_ERR_AIN_READ_BUFFER               HB_DEF_ERR(HB_ID_AIN, EN_ERR_AIN_READ_BUFFER)

#define HB_ERR_AOT_INVALID_PARAM             HB_DEF_ERR(HB_ID_AOT, EN_ERR_AOT_INVALID_PARAM)
#define HB_ERR_AOT_INVALID_DEVID             HB_DEF_ERR(HB_ID_AOT, EN_ERR_AOT_INVALID_DEVID)
#define HB_ERR_AOT_NOT_ALLOWED               HB_DEF_ERR(HB_ID_AOT, EN_ERR_AOT_NOT_ALLOWED)
#define HB_ERR_AOT_INVALID_CHN               HB_DEF_ERR(HB_ID_AOT, EN_ERR_AOT_INVALID_CHN)
#define HB_ERR_AOT_WRITE_BUFFER              HB_DEF_ERR(HB_ID_AOT, EN_RR_AOT_WRITE_BUFFER)


#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* __HB_AUDIO_ERROR_H__ */
